package flags

import "fmt"

type Flag interface {
	Name() string
	ShortName() string
	Description() string
	Parse(raw string) error
}

func Names(f Flag) string {
	names := ""
	if f.ShortName() != "" {
		names += fmt.Sprintf("-%s, ", f.ShortName())
	}
	names += fmt.Sprintf("--%s", f.Name())
	return names
}

func HelpLine(f Flag, paddedWidth int) string {
	argNames := ""
	if f.ShortName() != "" {
		argNames += fmt.Sprintf("-%s, ", f.ShortName())
	}
	argNames += fmt.Sprintf("--%s", f.Name())

	spaceSize := paddedWidth - len(argNames)
	if spaceSize > 0 {
		argNames += fmt.Sprintf("%*s", spaceSize, "")
	}

	return argNames + "  " + f.Description()
}
