package flags

import "strconv"

type FloatFlag struct {
	name        string
	shortName   string
	description string
	value       float64
}

var _ Flag = (*FloatFlag)(nil)

func Float(name, shortName string, defaultValue float64, description string) *FloatFlag {
	return &FloatFlag{
		name:        name,
		shortName:   shortName,
		description: description,
		value:       defaultValue,
	}
}

// Description implements Flag.
func (f *FloatFlag) Description() string {
	return f.description
}

// Name implements Flag.
func (f *FloatFlag) Name() string {
	return f.name
}

// ShortName implements Flag.
func (f *FloatFlag) ShortName() string {
	return f.shortName

}

// Value implements Flag.
func (f *FloatFlag) Parse(raw string) error {
	if raw == "" {
		return nil
	}

	floatVal, err := strconv.ParseFloat(raw, 64)
	if err != nil {
		return err
	}

	f.value = floatVal
	return nil
}

func (f *FloatFlag) Value() float64 {
	return f.value
}
